'''
Created on Mar 7, 2017

@author: fan
'''

import numpy as np
import scipy.interpolate as interpolate
import statsmodels.api as sm


def exp_value_interpolate_bp(prod_inst, util_opti,
                             b_ssv_sd, k_ssv_sd, epsilon_ssv_sd,
                             b_ssv, k_ssv, epsilon_ssv,
                             b_ssv_zr, k_ssv_zr, epsilon_ssv_zr,
                             states_vfi_dim, shocks_vfi_dim):
    """interpolate value function and expected value function. 
        
    Need three matrix here:
    1. state matrix x shock matrix where optimal choices were solved at
        - previously, shock for this = 0, but now shock vector might not be zero
    2. state matrix x shock matrix where shocks are drawn monte carlo way to allow 
        for averaging, integrating over shocks for each x row
    3. state matrix alone, shock = 0, each of the x row in matrix x
    
    """

    'A Get States to Integrate over'
    k_alpha_ae_sd, b_ssv_sd, \
    k_alpha_ae, b_ssv, \
    k_alpha_ae_zr, b_ssv_zr = \
        inter_states_bp(prod_inst, util_opti,
                        b_ssv_sd, k_ssv_sd, epsilon_ssv_sd,
                        b_ssv, k_ssv, epsilon_ssv,
                        b_ssv_zr, k_ssv_zr, epsilon_ssv_zr,
                        states_vfi_dim, shocks_vfi_dim)

    'B. invoke'
    util_emax = \
        exp_value_interpolate_main(u1=util_opti,
                                   x1=k_alpha_ae_sd, y1=b_ssv_sd,
                                   x2=k_alpha_ae, y2=b_ssv,
                                   x2_noshk=k_alpha_ae_zr, y2_noshk=b_ssv_zr,
                                   states_dim=states_vfi_dim, shocks_dim=shocks_vfi_dim,
                                   return_uxy=False)
    'C. collect'
    interpolant_exp_v = {'evu': util_emax, 'kae': k_alpha_ae_zr, 'b': b_ssv_zr}

    return interpolant_exp_v


def inter_states_bp(prod_inst, util_opti,
                    b_ssv_sd, k_ssv_sd, epsilon_ssv_sd,
                    b_ssv, k_ssv, epsilon_ssv,
                    b_ssv_zr, k_ssv_zr, epsilon_ssv_zr,
                    states_vfi_dim, shocks_vfi_dim):
    """interpolate value function and expected value function. 
        
    Need three matrix here:
    1. state matrix x shock matrix where optimal choices were solved at
        - previously, shock for this = 0, but now shock vector might not be zero
    2. state matrix x shock matrix where shocks are drawn monte carlo way to allow 
        for averaging, integrating over shocks for each x row
    3. state matrix alone, shock = 0, each of the x row in matrix x
    
    """

    'A. k_alpha_ae variables'
    k_alpha_sd = prod_inst.k_alpha(k_ssv_sd)
    k_alpha_ae_sd = prod_inst.k_alpha_ae(epsilon=epsilon_ssv_sd, k_alpha=k_alpha_sd)

    k_alpha = prod_inst.k_alpha(k_ssv)
    k_alpha_ae = prod_inst.k_alpha_ae(epsilon=epsilon_ssv, k_alpha=k_alpha)

    k_alpha_zr = prod_inst.k_alpha(k_ssv_zr)
    k_alpha_ae_zr = prod_inst.k_alpha_ae(epsilon=epsilon_ssv_zr, k_alpha=k_alpha_zr)

    #     y1 = b_ssv_sd
    #     y2 = b_ssv
    #     y3_noshk = b_ssv_zr

    return k_alpha_ae_sd, b_ssv_sd, k_alpha_ae, b_ssv, k_alpha_ae_zr, b_ssv_zr


def exp_value_interpolate_main(u1, x1, y1,
                               x2, y2,
                               x2_noshk, y2_noshk,
                               states_dim, shocks_dim,
                               return_uxy=False):
    #     interpolant_v(xnew, ynew)
    #         interpolant_v(1,2)
    #     interpolant_v([1,2],[3,4])
    #     interpolant_v(x2[0:3], y2[0:3])
    #
    #     xnew = np.arange(-5.01, 5.01, 1e-2)
    #     ynew = np.arange(-5.01, 5.01, 1e-2)
    #
    #     interpolate.griddata((x1,y1),u1, (x2,y2)).shape

    'A. Get Interpolant'
    #     u2 = interpolate.griddata((x1,y1), u1, (x2,y2), method='linear')
    u2 = interp_griddata(cur_u=u1, cur_x1=x1, cur_x2=y1,
                         new_x1=x2, new_x2=y2)
    'B2. Reshape u2, average over shocks'
    u2_mean = np.mean(np.reshape(u2, (int(states_dim), int(shocks_dim))), axis=1)

    """
    'A. Get Interpolant'    
    interpolant_v = interpolate.interp2d(x1, y1, u1)
    v_regress = regress(u1, regress_mat(x1, y1))
    
    'B1. Get value for another matrix'
    u2 = interpolant_v(x2, y2)
    
    'B2. Reshape u2, average over shocks'
    u2_mean = np.mean(np.reshape(u2, (states_dim, shocks_dim)), axis=1)
    
    'C. Interpolate again, at average values'
    interpolant_exp_v = interp2d(x2_noshk, y2_noshk, u2_mean)    
    exp_v_regress = regress(u2_mean, regress_mat(x2_noshk, y2_noshk))
    """

    'D. States and U Collection'
    if (return_uxy == True):
        uxy_coll = [[u1, x1, y1],
                    [u2, x2, y2],
                    [u2_mean, x2_noshk, y2_noshk]]
        return uxy_coll
    else:
        return u2_mean


def exp_value_interpolate_bpkp(hhp_inst, util_opti, b, k, b_shk, k_shk):
    """interpolate value function and expected value function. 
    
    cash and k_alpha calculation below does not repeat what happened already inside
    lifetimeutility. Inside lifetimeutility, we have next period cash and k_alpha
    here is this period    
    """

    # First Level Interpoalte
    cash, k_alpha = k_alpha_cash(hhp_inst, b, k)

    interpolant_v = interp2d(k_alpha, cash, util_opti)
    v_regress = regress(util_opti, regress_mat(k_alpha, cash))

    # Evaluate with shocks
    cash_shk, k_alpha_shks = k_alpha_cash(hhp_inst, b_shk, k_shk)
    util_opti_interp_shk = interpolant_v(k_alpha_shks, cash_shk)

    # Average over    
    util_opti_interp_shk_mean = np.mean(np.reshape(util_opti_interp_shk, \
                                                   (hhp_inst.states_dim, hhp_inst.shocks_dim)), axis=1)

    # Second Tier Interpolate
    interpolant_exp_v = interp2d(k_alpha, cash, util_opti_interp_shk_mean)
    exp_v_regress = regress(util_opti_interp_shk_mean, regress_mat(k_alpha, cash))

    return interpolant_exp_v, exp_v_regress


def k_alpha_cash(hhp_inst, b_vec, k_vec):
    cash_vec = hhp_inst.bdgt_inst.cash(b_vec, k_vec)
    k_alpha_vec = hhp_inst.prod_inst.k_alpha(k_vec)
    return cash_vec, k_alpha_vec


def interp_griddata(cur_u, cur_x1, cur_x2, new_x1, new_x2):
    """Centralize the invokation of 2D interpolation tool
    
    Potentially chagne this to something else if I don't like it. 
    """
    cur_x1 = np.ravel(cur_x1)
    cur_x2 = np.ravel(cur_x2)
    cur_u = np.ravel(cur_u)
    #     new_x1 = np.ravel(new_x1)
    #     new_x2 = np.ravel(new_x2)

    new_u = interpolate.griddata((cur_x1, cur_x2), cur_u,
                                 (new_x1, new_x2), method='linear')

    #     print('')
    #     print('cur_u:',np.min(cur_u),np.max(cur_u), cur_u.shape)
    #     print('cur_x1:',np.min(cur_x1),np.max(cur_x1),np.mean(cur_x1),cur_x1.shape)
    #     print('cur_x2:',np.min(cur_x2),np.max(cur_x2),np.mean(cur_x2),cur_x2.shape)
    #     print('new_x1:',np.min(new_x1),np.max(new_x1),np.mean(new_x1),new_x1.shape)
    #     print('new_x2:',np.min(new_x2),np.max(new_x2),np.mean(new_x2),new_x2.shape)
    #     print('new_u:',np.min(new_u),np.max(new_u),np.mean(new_u), new_u.shape)
    #     print('')

    return new_u


def interp2d(prod, cash, z=None, interpolant=None, kind='linear'):
    """Centralize the invokation of 2D interpolation tool
    
    Potentially chagne this to something else if I don't like it. 
    """

    if (interpolant == None):
        interpolant = interpolate.interp2d(
            prod.flatten(), cash.flatten(), z.flatten(), kind=kind)
        return interpolant
    else:
        evaluated = interpolant(prod.flatten(), cash.flatten())
        return evaluated


def interpRbf2D(prod, cash, z=None, interpolant=None, kind='linear'):
    #     https://stackoverflow.com/questions/37872171/how-can-i-perform-two-dimensional-interpolation-using-scipy

    if (interpolant == None):
        # default smooth=0 for interpolation
        interpolant = interpolate.Rbf(
            prod.flatten(), cash.flatten(), z.flatten(), function='cubic', smooth=0)

        return interpolant
    else:

        evaluated = interpolant(prod.flatten(), cash.flatten())
        return evaluated


def interpRbf3D(prod, cash, A, z=None, interpolant=None, kind='cubic'):
    #     https://stackoverflow.com/questions/37872171/how-can-i-perform-two-dimensional-interpolation-using-scipy

    if (interpolant == None):
        # default smooth=0 for interpolation
        interpolant = interpolate.Rbf(
            prod.flatten(), cash.flatten(), A.flatten(),
            z.flatten(), function='cubic', smooth=0)

        return interpolant
    else:

        evaluated = interpolant(prod.flatten(), cash.flatten(), A.flatten())
        return evaluated


def regress_mat(k_alpha, cash):
    if ((k_alpha is None) and (cash is None)):
        reg_mat_col_count = 3
        return reg_mat_col_count
    else:
        const = np.zeros(len(k_alpha)) + 1
        regress_mat = np.column_stack((const, k_alpha, cash))
        return regress_mat


def regress(dependent_var, rhs_var):
    model = sm.OLS(dependent_var, rhs_var)

    results = model.fit()
    beta = results.params
    tstats = results.tvalues

    return {'beta': beta, 'tstats': tstats}

# if __name__ == '__main__':
# 
#     import numpy as np
#     import time as time 
#     import scipy.interpolate as interpolate
#     
#     def func(x, y):
#         return x*(1-x)*np.cos(4*np.pi*x) * np.sin(4*np.pi*y**2)**2
#     
#     grid_x, grid_y = np.mgrid[0:1:100j, 0:1:200j]
#     
#     points = np.random.rand(1000, 2)
#     values = func(points[:,0], points[:,1])
#     
#     startTime = time.time()
#     for i in np.arange(1000):
#             
#         grid_z1 = interpolate.griddata(points, values, (grid_x, grid_y), method='linear')
#     t = time.time() - startTime
#     print(t)
#



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pyfan',
 'pyfan.amto',
 'pyfan.amto.array',
 'pyfan.amto.json',
 'pyfan.devel',
 'pyfan.devel.obj',
 'pyfan.gen',
 'pyfan.gen.rand',
 'pyfan.graph',
 'pyfan.graph.generic',
 'pyfan.graph.tools',
 'pyfan.panda.categorical',
 'pyfan.panda.inout',
 'pyfan.panda.sandbox',
 'pyfan.panda.stats',
 'pyfan.sandbox',
 'pyfan.stats.interpolate',
 'pyfan.stats.multinomial',
 'pyfan.util',
 'pyfan.util.inout',
 'pyfan.util.path',
 'pyfan.util.pdf',
 'pyfan.util.rmd',
 'pyfan.util.timer']

package_data = \
{'': ['*']}

install_requires = \
['cython>=0.29.20,<0.30.0',
 'matplotlib>=3.2.1,<4.0.0',
 'numba>=0.50.1,<0.51.0',
 'numpy>=1.18.5,<2.0.0',
 'python-frontmatter>=0.5.0,<0.6.0',
 'pyyaml>=5.3.1,<6.0.0',
 'scipy>=1.4.1,<2.0.0',
 'seaborn>=0.10.1,<0.11.0',
 'sklearn>=0.0,<0.1',
 'statsmodels>=0.11.1,<0.12.0']

setup_kwargs = {
    'name': 'pyfan',
    'version': '0.1.42',
    'description': '',
    'long_description': None,
    'author': 'Fan Wang',
    'author_email': 'wangfanbsg75@live.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



'''
Created on Aug 13, 2018

@author: fan
'''
import logging
import pandas as pd
import numpy as np
import pyfan.amto.json.json as support_json

logger = logging.getLogger(__name__)

cols = 5
rows = 20
np.random.seed(123)
data = (np.random.rand(rows, cols) - 0.5) * 100

df = pd.DataFrame(data, columns=['col' + str(ctr) for ctr in range(cols)])
winsor_coln_list = ['col0', 'col1', 'col3', 'col4']
coln_perc_cutoffs = {'col0': {'q_ge': 0, 'q_le': 0.9, 'v_ll': 10},
                     'col1': {'q_ge': 0.30, 'v_le': 50},
                     'col3': {'q_ge': 0.01, 'q_le': 0.60, 'v_ll': 40},
                     'col4': {'q_ge': 0.01, 'q_le': 1, 'v_ll': 33, 'v_gg': -5}}

return_type = 'winsorize'
# return_type = 'cutsubset'


'''
Deep copy original dataframe
'''
condi_ctr = 0
for winsor_coln in winsor_coln_list:
    col_perc_cutoff_dict = coln_perc_cutoffs[winsor_coln]

    '''
    Initialize cut min max
    '''
    winsorize_upper = np.Inf
    winsorize_lower = np.NINF

    '''
    Loop over all min and max conditions, including value and percentile conditions
    '''
    for key, val in col_perc_cutoff_dict.items():
        '''
        Current data series
        '''
        df_col = df[winsor_coln]

        '''
        Obtain Thresholds
        '''
        if ('q_' in key):
            '''
            Quantiles
            '''
            threshold = df_col.quantile(val)
        elif ('v_' in key):
            '''
            levels
            '''
            threshold = val
        else:
            raise ('did not find _q quantile or _v value in key')

        '''
        Generate conditions
        '''
        if ('_l' in key):
            winsorize_upper = np.minimum(winsorize_upper, threshold)
            if ('_le' in key):
                if_condi = (df_col <= threshold)
            elif ('_ll' in key):
                if_condi = (df_col < threshold)
            else:
                raise ('key:' + key + ', is invalid')

        elif ('_g' in key):
            winsorize_lower = np.maximum(winsorize_lower, threshold)
            if ('_ge' in key):
                if_condi = (df_col >= threshold)
            elif ('_gg' in key):
                if_condi = (df_col > threshold)
            else:
                raise ('key:' + key + ', is invalid')

        else:
            raise ('key:' + key + ', is invalid')

        '''
        Combine Conditions
        '''
        if (condi_ctr == 0):
            if_condi_all = if_condi
        else:
            if_condi_all = if_condi_all & if_condi
        condi_ctr = condi_ctr + 1

    '''
    Winsorize
    '''
    if (return_type == 'winsorize'):
        df[winsor_coln] = df_col.clip(lower=winsorize_lower, upper=winsorize_upper)

'''
Cut to data subset
'''
if (return_type == 'cutsubset'):
    df_return = df[if_condi_all]
    print(df_return)
    support_json.jdump(df_return.to_dict(), 'conditions:' + str(if_condi_all),
                       logger=logger.debug, print_here=True)

if (return_type == 'winsorize'):
    print(df)
    support_json.jdump(df.to_dict(), 'winsorized',
                       logger=logger.debug, print_here=True)



'''
Created on Aug 13, 2018

@author: fan

import panda.cutting as pd_cut

'''

import logging
logger = logging.getLogger(__name__)
import pyfan.amto.json.json as support_json
import pandas as pd
import numpy as np

# https://stackoverflow.com/questions/42217848/how-to-clip-pandas-dataframe-column-wise
 
def pd_winsorize_columnwise(df, winsor_coln_list,
                            coln_perc_cutoffs, return_type, 
                            print_array=False, json_debug=False):
    """
    Winsorizing column by column, no dependence across coluns. 
    Winsorize column by column
     
    cols = 5
    rows = 20
    np.random.seed(123)
    data = (np.random.rand(rows,cols)-0.5)*100
    
    df = pd.DataFrame(data, columns=['col' + str(ctr) for ctr in range(cols)])
    winsor_coln_list = ['col0', 'col1','col3','col4']
     
    Parameters
    ----------
    df: dataFrame
        initial dataset
    winsor_coln_list: list 
        list of column names to winsorize
        ['col0', 'col1','col3','col4']
    coln_perc_cutoffs: dictionary
        a nested dictionary where keys are elements of winsor_coln_list, and values
        are a dictionary with min and max percentiles of winsorizing values. 
        if min is 0, do not create cutcolss
         {'col0':{'q_ge':0,'q_le':0.9, 'v_ll':10},
          'col1':{'q_ge':0.30,'v_le':50},
          'col3':{'q_ge':0.01,'q_le':0.60, 'v_ll':40},
          'col4':{'q_ge':0.01,'q_le':1, 'v_ll':33, 'v_gg':-5}}
    return_type: string
         'winsorize' or 'cutsubset'
    """
    
    '''
    Deep copy original dataframe
    '''    
    condi_ctr = 0    
    for winsor_coln in winsor_coln_list:
        col_perc_cutoff_dict = coln_perc_cutoffs[winsor_coln]
        
        '''
        Initialize cut min max
        '''
        winsorize_upper = np.Inf
        winsorize_lower = np.NINF
        
        '''
        Loop over all min and max conditions, including value and percentile conditions
        '''
        for key, val in col_perc_cutoff_dict.items():        
            '''
            Current data series
            '''
            df_col = df[winsor_coln]
            
            '''
            Obtain Thresholds
            '''
            if ('q_' in key):
                '''
                Quantiles
                '''
                threshold = df_col.quantile(val)            
            elif ('v_' in key):
                '''
                levels
                '''
                threshold = val
            else:
                raise('did not find _q quantile or _v value in key')
            
            '''
            Generate conditions
            '''
            if ('_l' in key):
                winsorize_upper = np.minimum(winsorize_upper, threshold)            
                if ('_le' in key):
                    if_condi = (df_col <= threshold)
                elif ('_ll' in key):
                    if_condi = (df_col < threshold)
                else:
                    raise('key:'+ key + ', is invalid')
                    
            elif ('_g' in key):
                winsorize_lower = np.maximum(winsorize_lower, threshold)            
                if ('_ge' in key):
                    if_condi = (df_col >= threshold)
                elif ('_gg' in key):
                    if_condi = (df_col > threshold)                
                else:
                    raise('key:'+ key + ', is invalid')
    
            else:
                raise('key:'+ key + ', is invalid')
            
            '''
            Combine Conditions
            '''
            if (condi_ctr == 0):
                if_condi_all = if_condi
            else:
                if_condi_all = if_condi_all & if_condi
            condi_ctr = condi_ctr + 1
    
        '''
        Winsorize
        '''
        if(return_type == 'winsorize'):
            df[winsor_coln] = df_col.clip(lower=winsorize_lower, upper=winsorize_upper)
            
    '''
    Cut to data subset
    '''
    if(return_type == 'cutsubset'):
        df_return = df[if_condi_all]
        if (print_array):
            print(df_return)
        if (json_debug):
            support_json.jdump(df_return.to_dict(), 'conditions:' + str(if_condi_all),
                               logger=logger.debug, print_here = True)
        return df_return
    
    elif(return_type == 'winsorize'):
        if (print_array):
            print(df)
        if (json_debug):
            support_json.jdump(df.to_dict(), 'winsorized',
                               logger=logger.debug, print_here = True)    
        return df
    else:
        raise('wrong return_type')     


def sample_run():

    cols = 5
    rows = 20
    np.random.seed(123)
    data = (np.random.rand(rows,cols)-0.5)*100
    
    df = pd.DataFrame(data, columns=['col' + str(ctr) for ctr in range(cols)])
    winsor_coln_list = ['col0', 'col1','col3','col4']
    coln_perc_cutoffs = {'col0':{'q_ge':0,'q_le':0.9, 'v_ll':10},
                         'col1':{'q_ge':0.30,'v_le':50},
                         'col3':{'q_ge':0.01,'q_le':0.60, 'v_ll':40},
                         'col4':{'q_ge':0.01,'q_le':1, 'v_ll':33, 'v_gg':-5}}
    
    return_type = 'winsorize'
    # return_type = 'cutsubset'
    
    
    '''
    Deep copy original dataframe
    '''    
    condi_ctr = 0    
    for winsor_coln in winsor_coln_list:
        col_perc_cutoff_dict = coln_perc_cutoffs[winsor_coln]
        
        '''
        Initialize cut min max
        '''
        winsorize_upper = np.Inf
        winsorize_lower = np.NINF
        
        '''
        Loop over all min and max conditions, including value and percentile conditions
        '''
        for key, val in col_perc_cutoff_dict.items():        
            '''
            Current data series
            '''
            df_col = df[winsor_coln]
            
            '''
            Obtain Thresholds
            '''
            if ('q_' in key):
                '''
                Quantiles
                '''
                threshold = df_col.quantile(val)            
            elif ('v_' in key):
                '''
                levels
                '''
                threshold = val
            else:
                raise('did not find _q quantile or _v value in key')
            
            '''
            Generate conditions
            '''
            if ('_l' in key):
                winsorize_upper = np.minimum(winsorize_upper, threshold)            
                if ('_le' in key):
                    if_condi = (df_col <= threshold)
                elif ('_ll' in key):
                    if_condi = (df_col < threshold)
                else:
                    raise('key:'+ key + ', is invalid')
                    
            elif ('_g' in key):
                winsorize_lower = np.maximum(winsorize_lower, threshold)            
                if ('_ge' in key):
                    if_condi = (df_col >= threshold)
                elif ('_gg' in key):
                    if_condi = (df_col > threshold)                
                else:
                    raise('key:'+ key + ', is invalid')
    
            else:
                raise('key:'+ key + ', is invalid')
            
            '''
            Combine Conditions
            '''
            if (condi_ctr == 0):
                if_condi_all = if_condi
            else:
                if_condi_all = if_condi_all & if_condi
            condi_ctr = condi_ctr + 1
    
        '''
        Winsorize
        '''
        if(return_type == 'winsorize'):
            df[winsor_coln] = df_col.clip(lower=winsorize_lower, upper=winsorize_upper)
            
    '''
    Cut to data subset
    '''
    if(return_type == 'cutsubset'):
        df_return = df[if_condi_all]
        print(df_return)
        support_json.jdump(df_return.to_dict(), 'conditions:' + str(if_condi_all),
                           logger=logger.debug, print_here = True)
    
    if(return_type == 'winsorize'):
        print(df)
        support_json.jdump(df.to_dict(), 'winsorized',
                           logger=logger.debug, print_here = True)    



import pathlib
import os
import shutil
from distutils.dir_util import copy_tree


def fp_agg_move_subfiles(spt_root_src="C:/Users/fan/pyfan/vig/support/inout/_folder/fd/faa/",
                         st_srt_srh="_images",
                         st_fle_srh="*",
                         srt_agg='img',
                         ls_srt_dest=['C:/Users/fan/pyfan/vig/support/inout/_folder/fd/faa/',
                                      'C:/Users/fan/pyfan/vig/support/inout/_folder/'],
                         bl_delete_src=True,
                         bl_test=True,
                         verbose=False):
    """Aggregate and Move a Collection of Non-empty Folders

    A program (forexample mlx to tex conversion) creates in a folder a number
    of subfolder that stores images. Aggregate all the various image folders into
    a common image folder. And then move this common image folder to other destinations
    in order to flexibly generate aggregation files with common path that rely on
    images from various subfolders.

    Parameters
    ----------
    spt_root_src: string
        root folder where subfolders are contained
    st_srt_srh: string
        gather subfolder names that contain this string
    st_fle_srh: string
        search in subfolders for files whose name contain string
    srt_agg: string
        name of subfolder where found folders are aggregated at
    ls_srt_dest: :obj:`list` of :obj:`str`
        list of folder paths to move aggregate subfolders over to
    bl_delete_src: bool
        delete folders at existing locations
    bl_test: bool
        test by searching for paths dest and src, do not move
    verbose: bool
        print details

    Returns
    -------
    None
        nothing is returned

    Examples
    --------

    >>> fp_agg_move_subfiles(spt_root_src="C:/Users/fan/Math4Econ/matrix_application/",
    >>> 					 st_srt_srh="_images",
    >>> 					 st_fle_srh="*.png",
    >>> 					 srt_agg='img',
    >>> 					 ls_srt_dest=["C:/Users/fan/Math4Econ/"],
    >>> 					 bl_delete_src=False,
    >>> 					 bl_test=False,
    >>> 					 verbose=False)

    """

    # 1. Gather Folders
    ls_ls_srt_found = [[spt_root_src + spt, spt]
                       for spt in os.listdir(spt_root_src)
                       if ((st_srt_srh in spt)
                           and
                           (len([spn for spn
                                 in pathlib.Path(spt_root_src + spt).rglob(st_fle_srh)]) > 0))]
    if verbose:
        print(ls_ls_srt_found)

    # 2 Loop over destination folders, loop over source folders
    for srt in ls_srt_dest:

        # Move each folder over
        for ls_srt_found in ls_ls_srt_found:
            # Paths
            srt_source = ls_srt_found[0]
            srt_dest_subfolder = ls_srt_found[1]
            srt_dest = os.path.join(srt, srt_agg, srt_dest_subfolder)

            # print if verbose or testing
            if verbose or bl_test:
                print('srt_source:' + srt_source + ', srt_dest:' + srt_dest)

            # if not testing
            if not bl_test:
                # dest folders
                pathlib.Path(srt_dest).mkdir(parents=True, exist_ok=True)
                # move
                copy_tree(srt_source, srt_dest)

    # 3. Delete Sources
    if bl_delete_src and not bl_test:
        for ls_srt_found in ls_ls_srt_found:
            shutil.rmtree(ls_srt_found[0])


if __name__ == '__main__':
    # spt_root_src_u = "C:/Users/fan/Math4Econ/matrix_application/"
    # st_srt_srh_u = "_images"
    # st_fle_srh_u = "*.png"
    # srt_agg_u = 'img'
    # ls_srt_dest_u = [spt_root_src_u,
    #                  "C:/Users/fan/Math4Econ/"]
    # bl_delete_src_u = False
    # bl_test_u = False
    # verbose_u = False
    # fp_agg_move_subfiles(spt_root_src=spt_root_src_u,
    #                      st_srt_srh=st_srt_srh_u,
    #                      st_fle_srh=st_fle_srh_u,
    #                      srt_agg=srt_agg_u,
    #                      ls_srt_dest=ls_srt_dest_u,
    #                      bl_delete_src=bl_delete_src_u,
    #                      bl_test=bl_test_u,
    #                      verbose=verbose_u)

    spt_root_src_u = "C:/Users/fan/Math4Econ/matrix_application/"
    st_srt_srh_u = "twogoods_images"
    st_fle_srh_u = "*.png"
    srt_agg_u = 'img'
    ls_srt_dest_u = ['C:/Users/fan/Math4Econ/matrix_application/', 'C:/Users/fan/Math4Econ/']
    bl_delete_src_u = True
    bl_test_u = False
    verbose_u = True
    fp_agg_move_subfiles(spt_root_src=spt_root_src_u,
                         st_srt_srh=st_srt_srh_u,
                         st_fle_srh=st_fle_srh_u,
                         srt_agg=srt_agg_u,
                         ls_srt_dest=ls_srt_dest_u,
                         bl_delete_src=bl_delete_src_u,
                         bl_test=bl_test_u,
                         verbose=verbose_u)



'''
Created on Aug 9, 2018

@author: fan

Generate mean, and variance covariance of key state variables from data
'''
import logging
import numpy as np

logger = logging.getLogger(__name__)


def gen_mean(df, mean_var_list,
             short_mean_var_list=None,
             group_by_var_list=None,
             conditioning=None):
    if (short_mean_var_list == None):
        short_mean_var_list = mean_var_list

    '''
    Generate the means of a number of variables
    '''
    if (conditioning is not None):
        df_subset = df[conditioning]
    else:
        df_subset = df

    if (group_by_var_list is not None):
        means = df_subset.groupby(group_by_var_list)[mean_var_list].mean()
    elif (group_by_var_list is None):
        means = df_subset[mean_var_list].mean()

    means.info = {}
    means.info['obs'] = len(df_subset)
    means.info['means'] = {var_short: means[var] for var, var_short in zip(mean_var_list, short_mean_var_list)}

    return means


def gen_varcov(df, varcov_var_list,
               short_varcov_var_list=None,
               group_by_var_list=None,
               conditioning=None):
    if (short_varcov_var_list is None):
        short_varcov_var_list = varcov_var_list

    if (conditioning is not None):
        df_subset = df[conditioning]
    else:
        df_subset = df

    if (group_by_var_list is not None):
        varcov = df_subset.groupby(group_by_var_list)[varcov_var_list].cov()
    else:
        subset = df_subset[varcov_var_list]
        varcov = subset.cov()

    varcov.info = {}
    varcov.info['obs'] = len(df_subset)
    varcov.info['variance'] = {var_short: varcov[var][var] for var, var_short in
                               zip(varcov_var_list, short_varcov_var_list)}
    varcov.info['sd'] = {var_short: np.sqrt(varcov[var][var]) for var, var_short in
                         zip(varcov_var_list, short_varcov_var_list)}

    return varcov
